@@ -1,9 +1,31 @@
+# Copyright (c) 2008-2011 Tim Newsham, Andrey Mirtchovski
+# Copyright (c) 2011-2012 Peter V. Saveliev
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 """
 9P protocol implementation as documented in plan9 intro(5) and <fcall.h>.
 """
 
 import os
-import os.path
+import stat
 import sys
 import socket
 import select
@@ -154,6 +176,10 @@ class ClientError(Error):
     pass
 
 
+class VersionError(Error):
+    pass
+
+
 def modetostr(mode):
     bits = ["---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"]
 
@@ -167,6 +193,40 @@ def modetostr(mode):
     return "%s%s%s%s" % (d, b(6), b(3), b(0))
 
 
+def open2stat(mode):
+    return (mode & 3) |\
+            ((mode & OAPPEND) >> 4) |\
+            ((mode & OEXCL) >> 5) |\
+            ((mode & OTRUNC) << 5)
+
+
+def open2plan(mode):
+    return (mode & 3) |\
+            ((mode & os.O_APPEND) << 4) |\
+            ((mode & os.O_EXCL) << 5) |\
+            ((mode & os.O_TRUNC) >> 5)
+
+
+def mode2stat(mode):
+    return (mode & 0o777) |\
+            ((mode & DMDIR ^ DMDIR) >> 16) |\
+            ((mode & DMDIR) >> 17) |\
+            ((mode & DMSYMLINK) >> 10) |\
+            ((mode & DMSYMLINK) >> 12) |\
+            ((mode & DMSETUID) >> 8) |\
+            ((mode & DMSETGID) >> 8) |\
+            ((mode & DMSTICKY) >> 7)
+
+
+def mode2plan(mode):
+    return (mode & 0o777) | \
+            ((mode & stat.S_IFDIR) << 17) |\
+            ((mode & stat.S_ISUID) << 8) |\
+            ((mode & stat.S_ISGID) << 8) |\
+            ((mode & stat.S_ISVTX) << 7) |\
+            (int(mode == stat.S_IFLNK) << 25)
+
+
 def hash8(obj):
     return int(abs(hash(obj)))
 
@@ -456,15 +516,16 @@ class Server(object):
     A server interface to the protocol.
     Subclass this to provide service
     """
-    msize = 8192
     chatty = False
     readpool = []
     writepool = []
     activesocks = {}
 
     def __init__(self, listen, authmode=None, fs=None, user=None,
-            dom=None, key=None, chatty=False, dotu=False):
-        if authmode == None:
+            dom=None, key=None, chatty=False, dotu=False, msize=8192):
+        self.msize = msize
+
+        if authmode is None:
             self.authfs = None
         elif authmode == 'pki':
             import pki
@@ -628,7 +689,7 @@ class Server(object):
                 print >>sys.stderr, "error in respond: ", traceback.print_exc()
                 return -1
         else:
-            raise ServerError("can not handle message type " + \
+            raise ServerError("can not handle message type " +
                     cmdName[req.ifcall.type])
 
         req.ofcall.tag = req.ifcall.tag
@@ -681,7 +742,7 @@ class Server(object):
             except Exception, e:
                 if self.chatty:
                     print >>sys.stderr, traceback.print_exc()
-                self.respond(req, 'unhandled internal exception: ' + \
+                self.respond(req, 'unhandled internal exception: ' +
                         str(e.args[0]))
         else:
             self.respond(req, "unhandled message: %s" % (
@@ -737,15 +798,16 @@ class Server(object):
             req.ofcall.version = '9P2000'
             req.sock.marshal.dotu = 0
 
-        req.ofcall.msize = req.ifcall.msize
+        req.ofcall.msize = min(req.ifcall.msize, self.msize)
         self.respond(req, None)
 
     def rversion(self, req, error):
-        self.msize = req.ofcall.msize
+        # self.msize = req.ofcall.msize
+        pass
 
     def tauth(self, req):
-        if self.authfs == None:
-            self.respond(req, "%s: authentication not required" % \
+        if self.authfs is None:
+            self.respond(req, "%s: authentication not required" %
                     (sys.argv[0]))
             return
 
@@ -782,7 +844,7 @@ class Server(object):
                 return
             elif self.chatty:
                 print >>sys.stderr, "authenticated as %r" % req.ifcall.uname
-        elif self.authmode != None:
+        elif self.authmode is not None:
             self.respond(req, 'authentication not complete')
 
         req.fid.uid = req.ifcall.uname
@@ -851,7 +913,7 @@ class Server(object):
             self.respond(req, "no walk function")
 
     def rwalk(self, req, error):
-        if error or (len(req.ofcall.wqid) < len(req.ifcall.wname) and \
+        if error or (len(req.ofcall.wqid) < len(req.ifcall.wname) and
                 len(req.ifcall.wname) > 0):
             if req.ifcall.fid != req.ifcall.newfid and req.newfid:
                 req.sock.delfid(req.ifcall.newfid)
@@ -926,7 +988,7 @@ class Server(object):
         req.ofcall.iounit = self.msize - IOHDRSZ
 
     def bufread(self, req, buf):
-        req.ofcall.data = buf[req.ifcall.offset: req.ifcall.offset + \
+        req.ofcall.data = buf[req.ifcall.offset: req.ifcall.offset +
                 req.ifcall.count]
         return self.respond(req, None)
 
@@ -936,8 +998,8 @@ class Server(object):
             return self.respond(req, Eunknownfid)
         if req.ifcall.count < 0:
             return self.respond(req, Ebotch)
-        if req.ifcall.offset < 0 or ((req.fid.qid.type & QTDIR) and \
-                (req.ifcall.offset != 0) and \
+        if req.ifcall.offset < 0 or ((req.fid.qid.type & QTDIR) and
+                (req.ifcall.offset != 0) and
                 (req.ifcall.offset != req.fid.diroffset)):
             return self.respond(req, Ebadoffset)
         if req.fid.qid.type & QTAUTH and self.authfs:
@@ -990,7 +1052,7 @@ class Server(object):
             req.ifcall.count = self.msize - IOHDRSZ
         o = req.fid.omode & 3
         if o != OWRITE and o != ORDWR:
-            return self.respond(req, \
+            return self.respond(req,
                     "write on fid with open mode 0x%ux" % req.fid.omode)
         if hasattr(self.fs, 'write'):
             self.fs.write(self, req)
@@ -1004,7 +1066,7 @@ class Server(object):
         req.fid = req.sock.getfid(req.ifcall.fid)
         if not req.fid:
             return self.respond(req, Eunknownfid)
-        if hasattr(self.fs, 'clunk'):
+        if hasattr(self.fs, 'clunk') and not (req.fid.qid.type & QTAUTH):
             self.fs.clunk(self, req)
         else:
             self.respond(req, None)
@@ -1075,11 +1137,12 @@ class Client(object):
     F = 13
 
     path = ''  # for 'getwd' equivalent
-    msize = 8192
 
-    def __init__(self, fd, credentials, authsrv=None, chatty=0, dotu=0):
+    def __init__(self, fd, credentials, authsrv=None, chatty=0, dotu=0,
+            msize=8192):
         self.credentials = credentials
         self.dotu = dotu
+        self.msize = msize
         self.fd = Sock(fd, dotu, chatty)
         self.login(authsrv, credentials)
 
@@ -1206,9 +1269,10 @@ class Client(object):
             ver = versionu
         else:
             ver = version
-        fcall = self._version(8 * 1024, ver)
+        fcall = self._version(self.msize, ver)
+        self.msize = fcall.msize
         if fcall.version != ver:
-            raise ClientError("version mismatch: %r" % fcall.version)
+            raise VersionError("version mismatch: %r" % fcall.version)
 
         fcall.afid = self.AFID
         try:
@@ -1219,7 +1283,7 @@ class Client(object):
         if fcall.afid != NOFID:
             fcall.aqid = rfcall.aqid
 
-            if credentials.authmode == None:
+            if credentials.authmode is None:
                 raise ClientError('no authentication method')
             elif credentials.authmode == 'sk1':
                 import sk1
